@@ -27,17 +27,20 @@ import (
 	"time"
 
 	"github.com/dustin/go-humanize"
+	"github.com/pbnjay/memory"
 	"github.com/pmezard/go-difflib/difflib"
 	"modernc.org/cc/v4"
 	"modernc.org/ccorpus2"
 	"modernc.org/fileutil"
 	"modernc.org/gc/v2"
+	_ "modernc.org/libc"
 	"modernc.org/mathutil"
 )
 
 const (
 	csmithBitfields   = "--bitfields"    // --bitfields | --no-bitfields: enable | disable full-bitfields structs (enabled by default). // Was disabled by default in older versions,
 	csmithNoBitfields = "--no-bitfields" // --bitfields | --no-bitfields: enable | disable full-bitfields structs (enabled by default).
+
 )
 
 var (
@@ -46,24 +49,26 @@ var (
 	oCSmithClimit = flag.Duration("csmithc", 1*time.Minute, "")
 	oDebug        = flag.Bool("debug", false, "")
 	oErr1         = flag.Bool("err1", false, "first error line only")
-	oKeep         = flag.Bool("keep", false, "keep temp directories")
+	oKeep         = flag.Bool("keep", false, "keep temp directories (only with -work)")
+	oLibc         = flag.String("libc", "modernc.org/libc", "")
 	oPanic        = flag.Bool("panic", false, "panic on miscompilation")
-	oPin          = flag.String("pin", "0", "argument of -experiment-pin")
 	oShellTime    = flag.Duration("shelltimeout", 3600*time.Second, "shell() time limit")
 	oStackTrace   = flag.Bool("trcstack", false, "")
 	oTrace        = flag.Bool("trc", false, "print tested paths.")
-	oTraceC       = flag.Bool("trcc", false, "trace TestExec transiple errors")
+	oTraceC       = flag.Bool("trcc", false, "trace TestExec transpile errors")
 	oTraceCC      = flag.Bool("trccc", false, "trace TestExec C compiler errors")
 	oTraceF       = flag.Bool("trcf", false, "print test file content")
 	oTraceO       = flag.Bool("trco", false, "print test output")
-	oXWork        = flag.String("xwork", "", "TestExec will use a go.work file for packages in the CSV list")
 	oXTags        = flag.String("xtags", "", "passed to go build of TestSQLite")
+	oXWork        = flag.String("xwork", "", "TestExec will use a go.work file for packages in the CSV list")
 
-	cfs    fs.FS
-	goarch = runtime.GOARCH
-	goos   = runtime.GOOS
-	hostCC string
-	re     *regexp.Regexp
+	cfs         fs.FS
+	goarch      = runtime.GOARCH
+	goos        = runtime.GOOS
+	hostCC      string
+	re          *regexp.Regexp
+	target      = fmt.Sprintf("%s/%s", goos, goarch)
+	totalMemory = memory.TotalMemory()
 
 	csmithDefaultArgs = strings.Join([]string{
 		"--max-nested-struct-level", "10", // --max-nested-struct-level <num>: limit maximum nested level of structs to <num>(default 0). Only works in the exhaustive mode.
@@ -101,17 +106,6 @@ func (f *overlayFS) Open(name string) (fs.File, error) {
 }
 
 func TestMain(m *testing.M) {
-	switch runtime.GOOS {
-	case "linux":
-		switch runtime.GOARCH {
-		case "amd64", "386":
-			// ok
-		default:
-			panic(todo("unsupported target: %s/%s", runtime.GOOS, runtime.GOARCH)) //TODO
-		}
-	default:
-		panic(todo("unsupported target: %s/%s", runtime.GOOS, runtime.GOARCH)) //TODO
-	}
 	isTesting = true
 	testWD, err := filepath.Abs("testdata")
 	if err != nil {
@@ -168,6 +162,7 @@ func TestGoAlign(t *testing.T) {
 		"freebsd/386",
 		"freebsd/amd64",
 		"freebsd/arm",
+		"freebsd/arm64",
 		"linux/386",
 		"linux/amd64",
 		"linux/arm",
@@ -300,21 +295,8 @@ func inDir(dir string, f func() error) (err error) {
 	return f()
 }
 
-type echoWriter struct {
-	mu     sync.Mutex
-	w      bytes.Buffer
-	silent bool
-}
-
-func (w *echoWriter) Write(b []byte) (int, error) {
-	w.mu.Lock()
-
-	defer w.mu.Unlock()
-
-	if !w.silent {
-		os.Stderr.Write(b)
-	}
-	return w.w.Write(b)
+func shell(echo bool, cmd string, args ...string) ([]byte, error) {
+	return shell0(*oShellTime, echo, cmd, args...)
 }
 
 func TestExec(t *testing.T) {
@@ -332,9 +314,14 @@ func TestExec(t *testing.T) {
 
 		tmp = temp
 
-		defer func() {
-			os.RemoveAll(tmp)
-		}()
+		switch {
+		case *oKeep:
+			t.Logf("keeping dir %s\n", tmp)
+		default:
+			defer func() {
+				os.RemoveAll(tmp)
+			}()
+		}
 	default:
 		tmp = t.TempDir()
 	}
@@ -344,6 +331,10 @@ func TestExec(t *testing.T) {
 			return fmt.Errorf("%s\vFAIL: %v", out, err)
 		}
 
+		if out, err := shell(true, "go", "get", *oLibc+"@latest"); err != nil {
+			return fmt.Errorf("%s\vFAIL: %v", out, err)
+		}
+
 		switch s := *oXWork; {
 		case s != "":
 			if out, err := shell(true, "go", "work", "init"); err != nil {
@@ -359,10 +350,6 @@ func TestExec(t *testing.T) {
 					return fmt.Errorf("%s\vFAIL: %v", out, err)
 				}
 			}
-		default:
-			if out, err := shell(true, "go", "get", defaultLibc+"@master"); err != nil { //TODO- &master
-				return fmt.Errorf("%s\vFAIL: %v", out, err)
-			}
 		}
 
 		for _, v := range []struct {
@@ -433,8 +420,14 @@ func testExec(t *testing.T, cfsDir string, exec bool, g *golden) {
 		}
 
 		p.file()
+		base := filepath.Base(path)
 		switch {
-		case re != nil && !re.MatchString(filepath.Base(path)):
+		case re != nil && !re.MatchString(base):
+			p.skip()
+			return nil
+		}
+
+		if totalMemory < 4<<30 && strings.HasPrefix(base, "limits-") {
 			p.skip()
 			return nil
 		}
@@ -459,50 +452,6 @@ func trccc(path string, err error) {
 	}
 }
 
-func shell(echo bool, cmd string, args ...string) ([]byte, error) {
-	cmd, err := exec.LookPath(cmd)
-	if err != nil {
-		return nil, err
-	}
-
-	wd, err := absCwd()
-	if err != nil {
-		return nil, err
-	}
-
-	if echo {
-		fmt.Printf("execute %s %q in %s\n", cmd, args, wd)
-	}
-	var b echoWriter
-	b.silent = !echo
-	ctx, cancel := context.WithTimeout(context.Background(), *oShellTime)
-	defer cancel()
-	c := exec.CommandContext(ctx, cmd, args...)
-	c.Stdout = &b
-	c.Stderr = &b
-	c.WaitDelay = *oShellTime + time.Minute
-	err = c.Start()
-	if err != nil {
-		return nil, err
-	}
-
-	err = c.Wait()
-	return b.w.Bytes(), err
-}
-
-func absCwd() (string, error) {
-	wd, err := os.Getwd()
-	if err != nil {
-		return "", err
-	}
-
-	if wd, err = filepath.Abs(wd); err != nil {
-		return "", err
-	}
-
-	return wd, nil
-}
-
 func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *golden, id int, args []string) (err error) {
 	defer func() {
 		if err != nil {
@@ -514,21 +463,22 @@ func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *go
 	var cCompilerFailed, cExecFailed bool
 	ofn := fmt.Sprint(id)
 	bin := "cbin_" + enforceBinaryExt(ofn)
+
+	defer func(nm string) { os.Remove(nm) }(bin)
+
 	switch {
 	case !execute:
-		if _, err = shell(false, hostCC, "-c", "-w", path, "-lm"); err != nil {
+		if _, err = shell(false, hostCC, "-c", "-w", path, "-lm", "-lpthread"); err != nil {
 			cCompilerFailed = true
 			trccc(path, err)
 		}
 	default:
-		if _, err = shell(false, hostCC, "-o", bin, "-w", path, "-lm"); err != nil {
+		if _, err = shell(false, hostCC, "-o", bin, "-w", path, "-lm", "-lpthread"); err != nil {
 			trccc(path, err)
 			cCompilerFailed = true
 		}
 	}
 
-	defer os.Remove(ofn)
-
 	cbinRC := -1
 	var cOut []byte
 	if execute && !cCompilerFailed {
@@ -543,7 +493,12 @@ func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *go
 
 	ofn += ".go"
 
-	defer os.Remove(ofn)
+	switch {
+	case *oKeep:
+		t.Logf("keeping file %s\n", ofn)
+	default:
+		defer os.Remove(ofn)
+	}
 
 	var out bytes.Buffer
 	switch {
@@ -553,12 +508,13 @@ func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *go
 			goarch,
 			[]string{
 				"ccgo",
+
 				"-c",
 				"-verify-types",
 				"--prefix-field=F",
-				"-ignore-unsupported-atomic-sizes",
 				"-ignore-vector-functions",
-				"-experiment-pin", *oPin,
+				"-keep-object-files",
+				// "--libc", *oLibc,
 				path,
 			},
 			&out, &out, nil).Main()
@@ -568,12 +524,15 @@ func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *go
 			goarch,
 			[]string{
 				"ccgo",
+
 				"-o", ofn,
 				"-verify-types",
 				"--prefix-field=F",
 				"-ignore-vector-functions",
-				"-ignore-unsupported-atomic-sizes",
-				"-experiment-pin", *oPin,
+				"-keep-object-files",
+				"-positions",
+				"-full-paths",
+				// "--libc", *oLibc,
 				path,
 			},
 			&out, &out, nil).Main()
@@ -604,8 +563,15 @@ func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *go
 	}
 
 	bin = "gobin_" + enforceBinaryExt(ofn)
+
+	defer func(nm string) { os.Remove(nm) }(bin)
+
 	var shOut []byte
-	if shOut, err = shell(false, "go", "build", "-o", bin, ofn); err != nil {
+	buildArgs := []string{"build"}
+	if s := *oXTags; s != "" {
+		buildArgs = append(buildArgs, fmt.Sprintf("-tags=%s", s))
+	}
+	if shOut, err = shell(false, "go", append(buildArgs, "-o", bin, ofn)...); err != nil {
 		// trc("gc %v %v", path, err)
 		if cCompilerFailed || isTestExecKnownFail(fullPath) {
 			p.skip()
@@ -633,7 +599,7 @@ func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *go
 				return nil
 			}
 
-			err := errorf("%s: %s: FAIL: %v", fullPath, goOut, err)
+			err := errorf("%s: %s: FAIL: %v\ngobinRC=%vcbinRC=%v", fullPath, goOut, err, gobinRC, cbinRC)
 			if *oPanic {
 				panic(err)
 			}
@@ -643,12 +609,16 @@ func testExec1(t *testing.T, p *parallel, root, path string, execute bool, g *go
 				return nil
 			}
 
-			trc("`%s`: {}, // EXEC FAIL: %v", fullPath, firstError(err, true))
+			trc("`%s`: {}, // EXEC FAIL: %v", fullPath, firstError(err, *oErr1))
 			p.err(err)
 			return firstError(err, *oErr1)
 		}
 	}
 
+	// ofile, _ := os.ReadFile(ofn) //TODO-DBG
+	// trc("\n%s", ofile)           //TODO-DBG
+	// trc("\n%s", goOut)           //TODO-DBG
+
 	cOut = bytes.TrimSpace(cOut)
 	goOut = bytes.TrimSpace(goOut)
 	if *oTraceO {
@@ -805,13 +775,7 @@ func TestCSmith(t *testing.T) {
 
 	csmith, err := exec.LookPath("csmith")
 	if err != nil {
-		switch runtime.GOOS {
-		case "windows": //TODO
-			t.Skip(err)
-			return
-		default:
-			t.Fatal(err)
-		}
+		t.Skip(err)
 	}
 
 	bigEndian := abi.ByteOrder == binary.BigEndian
@@ -837,7 +801,7 @@ func TestCSmith(t *testing.T) {
 	defer func() {
 		switch {
 		case *oKeep:
-			fmt.Printf("%s kept\n", temp)
+			t.Logf("keeping dir %s\n", temp)
 		default:
 			os.RemoveAll(temp)
 		}
@@ -853,6 +817,10 @@ func TestCSmith(t *testing.T) {
 
 	switch s := *oXWork; {
 	case s != "":
+		if out, err := shell(true, "go", "work", "init"); err != nil {
+			t.Fatalf("%s\vFAIL: %v", out, err)
+		}
+
 		if out, err := shell(true, "go", "work", "use", "."); err != nil {
 			t.Fatalf("%s\vFAIL: %v", out, err)
 		}
@@ -863,7 +831,7 @@ func TestCSmith(t *testing.T) {
 			}
 		}
 	default:
-		if out, err := shell(true, "go", "get", defaultLibc+"@master"); err != nil { //TODO- @master
+		if out, err := shell(true, "go", "get", *oLibc+"@latest"); err != nil { //TODO- @latest
 			t.Fatalf("%s\vFAIL: %v", out, err)
 		}
 	}
@@ -915,6 +883,7 @@ func TestCSmith(t *testing.T) {
 		"--bitfields --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid --max-nested-struct-level 10 -s 1906742816",
 		"--bitfields --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid --max-nested-struct-level 10 -s 3629008936",
 		"--bitfields --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid --max-nested-struct-level 10 -s 612971101",
+		"--max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid --bitfields -s 1701143130",
 		"--max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid --bitfields -s 20004725738999789",
 		"--max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid --bitfields -s 3654957324",
 		"--no-bitfields --max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid -s 1302111308",
@@ -923,6 +892,9 @@ func TestCSmith(t *testing.T) {
 		"--no-bitfields --max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid -s 3720922579",
 		"--no-bitfields --max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid -s 4263172072",
 		"--no-bitfields --max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid -s 572192313",
+
+		//TODO linux/riscv64
+		"--max-nested-struct-level 10 --no-const-pointers --no-consts --no-packed-struct --no-volatile-pointers --no-volatiles --paranoid --bitfields -s 1714958724",
 	}
 	var ch <-chan time.Time
 	t0 := time.Now()
@@ -995,7 +967,7 @@ out:
 			csp = fmt.Sprintf("-I%s", s)
 		}
 
-		ccOut, err := exec.Command(hostCC, "-o", binaryName, "main.c", csp).CombinedOutput()
+		ccOut, err := exec.Command(hostCC, "-o", binaryName, "main.c", csp, "-lm", "-lpthread").CombinedOutput()
 		if err != nil {
 			t.Logf("%s\n%s\ncc: %v", extra, ccOut, err)
 			continue
@@ -1040,8 +1012,7 @@ out:
 				"--prefix-field=F",
 				"-ignore-asm-errors",
 				"-ignore-vector-functions",
-				"-ignore-unsupported-alignment",
-				"-experiment-pin", *oPin,
+				"--libc", *oLibc,
 				"main.c",
 				csp,
 			},
@@ -1107,19 +1078,21 @@ func durationStr(d time.Duration) string {
 }
 
 func TestSQLite(t *testing.T) {
-	switch runtime.GOARCH {
-	case "amd64", "386":
-		// ok
-	default:
-		t.Skip("TODO")
+	if goos == "windows" {
+		//	--- FAIL: TestSQLite (58.93s)
+		//	    --- FAIL: TestSQLite/simple (58.93s)
+		//	        all_test.go:1185: shell.o.go:64044:8: undefined: "IsValidCodePage" external (all_test.go:1190:testSQLiteSimple: all_test.go:1190:testSQLiteSimple: ccgo.go:194:Main: ccgo.go:642:main: link.go:302:link: link.go:860:link:)
+		//	            shell.o.go:64059:30: undefined: "GetConsoleOutputCP" external (all_test.go:1190:testSQLiteSimple: all_test.go:1190:testSQLiteSimple: ccgo.go:194:Main: ccgo.go:642:main: link.go:302:link: link.go:860:link:)
+		//	            shell.o.go:64061:3: undefined: "SetConsoleCP" external (all_test.go:1190:testSQLiteSimple: all_test.go:1190:testSQLiteSimple: ccgo.go:194:Main: ccgo.go:642:main: link.go:302:link: link.go:860:link:)
+		//	            shell.o.go:64063:3: undefined: "SetConsoleOutputCP" external (all_test.go:1190:testSQLiteSimple: all_test.go:1190:testSQLiteSimple: ccgo.go:194:Main: ccgo.go:642:main: link.go:302:link: link.go:860:link:)
+		//	            shell.o.go:64582:5: undefined: "_fstat64i32" external (all_test.go:1190:testSQLiteSimple: all_test.go:1190:testSQLiteSimple: ccgo.go:194:Main: ccgo.go:642:main: link.go:302:link: link.go:860:link:)
+		t.Skip("disabled on Windows")
 	}
-
 	t.Run("simple", testSQLiteSimple)
 	t.Run("speedtest1", testSQLiteSpeedTest1)
 }
 
 func testSQLiteSimple(t *testing.T) {
-	t.Skip("TODO")
 	const (
 		dir  = "assets/sqlite-amalgamation"
 		main = "main.go"
@@ -1138,7 +1111,7 @@ func testSQLiteSimple(t *testing.T) {
 
 	switch {
 	case *oKeep:
-		t.Log(temp)
+		t.Logf("keeping dir %s\n", temp)
 	default:
 		defer os.RemoveAll(temp)
 	}
@@ -1147,38 +1120,7 @@ func testSQLiteSimple(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	ccgoArgs := []string{
-		"ccgo",
-
-		"-DHAVE_USLEEP",
-		"-DLONGDOUBLE_TYPE=double",
-		"-DSQLITE_DEBUG",
-		"-DSQLITE_DEFAULT_MEMSTATUS=0",
-		"-DSQLITE_ENABLE_DBPAGE_VTAB",
-		"-DSQLITE_LIKE_DOESNT_MATCH_BLOBS",
-		"-DSQLITE_MEMDEBUG",
-		"-DSQLITE_THREADSAFE=0",
-		"--prefix-field=F",
-		"-positions",
-		"-full-paths",
-		"-verify-types",
-		"-ignore-unsupported-alignment",
-		"-ignore-unsupported-atomic-sizes",
-		"-ignore-vector-functions",
-		"-experiment-pin", *oPin,
-		"-o", main,
-		filepath.Join(dir, "shell.c"),
-		filepath.Join(dir, "sqlite3.c"),
-		filepath.Join(dir, "patch.c"),
-	}
-	if *oKeep {
-		ccgoArgs = append(ccgoArgs, "-keep-object-files", "-extended-errors", "-debug-linker-save")
-	}
-	if *oDebug {
-		ccgoArgs = append(ccgoArgs, "-DSQLITE_DEBUG_OS_TRACE", "-DSQLITE_FORCE_OS_TRACE", "-DSQLITE_LOCK_TRACE")
-	}
-
-	if out, err := shell(true, "go", "mod", "init", "example.com/ccgo/v4/lib/sqlite1"); err != nil {
+	if out, err := shell(true, "go", "mod", "init", "example.com/ccgo/v4/lib/sqlitea"); err != nil {
 		t.Fatalf("%v\n%s", err, out)
 	}
 
@@ -1198,11 +1140,46 @@ func testSQLiteSimple(t *testing.T) {
 			}
 		}
 	default:
-		if out, err := shell(true, "go", "get", defaultLibc+"@master"); err != nil { //TODO- &master
+		if out, err := shell(true, "go", "get", *oLibc+"@latest"); err != nil {
 			t.Fatalf("%s\vFAIL: %v", out, err)
 		}
 	}
 
+	ccgoArgs := []string{
+		"ccgo",
+
+		"-DHAVE_USLEEP",
+		"-DLONGDOUBLE_TYPE=double",
+		// "-DSQLITE_DEBUG",
+		"-DSQLITE_DEFAULT_MEMSTATUS=0",
+		"-DSQLITE_ENABLE_DBPAGE_VTAB",
+		"-DSQLITE_LIKE_DOESNT_MATCH_BLOBS",
+		// "-DSQLITE_MEMDEBUG",
+		"-DSQLITE_THREADSAFE=0",
+		"-DSQLITE_WITHOUT_ZONEMALLOC",
+		"--prefix-field=F",
+		"-positions",
+		"-full-paths",
+		"-verify-types",
+		"-ignore-vector-functions",
+		"-ignore-vector-functions",
+		"--libc", *oLibc,
+		"-o", main,
+		filepath.Join(dir, "shell.c"),
+		filepath.Join(dir, "sqlite3.c"),
+		filepath.Join(dir, "patch.c"),
+	}
+	switch target {
+	case "darwin/arm64", "linux/arm64", "freebsd/arm64":
+		ccgoArgs = append(ccgoArgs, "-ignore-unsupported-alignment")
+	}
+	if *oKeep {
+		ccgoArgs = append(ccgoArgs, "-keep-object-files", "-extended-errors", "-debug-linker-save")
+	}
+	if *oDebug {
+		ccgoArgs = append(ccgoArgs, "-DSQLITE_DEBUG_OS_TRACE", "-DSQLITE_FORCE_OS_TRACE", "-DSQLITE_LOCK_TRACE")
+	}
+
 	if !func() (r bool) {
 		defer func() {
 			if err := recover(); err != nil {
@@ -1238,7 +1215,7 @@ func testSQLiteSimple(t *testing.T) {
 
 	shell := "./shell"
 	if runtime.GOOS == "windows" {
-		shell = "shell.exe"
+		shell = "./shell.exe"
 	}
 	args := []string{"build"}
 	if s := *oXTags; s != "" {
@@ -1311,7 +1288,7 @@ func testSQLiteSpeedTest1(t *testing.T) {
 
 	switch {
 	case *oKeep:
-		t.Log(temp)
+		t.Logf("keeping dir %s\n", temp)
 	default:
 		defer os.RemoveAll(temp)
 	}
@@ -1325,32 +1302,35 @@ func testSQLiteSpeedTest1(t *testing.T) {
 
 		"-DHAVE_USLEEP",
 		"-DLONGDOUBLE_TYPE=double",
-		"-DSQLITE_DEBUG",
+		// "-DSQLITE_DEBUG",
 		"-DSQLITE_DEFAULT_MEMSTATUS=0",
 		"-DSQLITE_ENABLE_DBPAGE_VTAB",
 		"-DSQLITE_LIKE_DOESNT_MATCH_BLOBS",
-		"-DSQLITE_MEMDEBUG",
+		// "-DSQLITE_MEMDEBUG",
 		"-DSQLITE_THREADSAFE=0",
+		"-DSQLITE_WITHOUT_ZONEMALLOC",
 		"--prefix-field=F",
 		"-positions",
 		"-full-paths",
 		"-verify-types",
-		"-ignore-unsupported-alignment",
-		"-ignore-unsupported-atomic-sizes",
 		"-ignore-vector-functions",
-		"-experiment-pin", *oPin,
+		"--libc", *oLibc,
 		"-o", main,
 		filepath.Join(dir, "speedtest1.c"),
 		filepath.Join(dir, "sqlite3.c"),
 		filepath.Join(dir, "patch.c"),
 	}
+	switch target {
+	case "darwin/arm64", "freebsd/arm64":
+		ccgoArgs = append(ccgoArgs, "-ignore-unsupported-alignment")
+	}
 	if *oKeep {
 		ccgoArgs = append(ccgoArgs, "-keep-object-files", "-extended-errors", "-debug-linker-save")
 	}
 	if *oDebug {
 		ccgoArgs = append(ccgoArgs, "-DSQLITE_DEBUG_OS_TRACE", "-DSQLITE_FORCE_OS_TRACE", "-DSQLITE_LOCK_TRACE")
 	}
-	if out, err := shell(true, "go", "mod", "init", "example.com/ccgo/v4/lib/sqlite1"); err != nil {
+	if out, err := shell(true, "go", "mod", "init", "example.com/ccgo/v4/lib/sqliteb"); err != nil {
 		t.Fatalf("%v\n%s", err, out)
 	}
 
@@ -1370,7 +1350,7 @@ func testSQLiteSpeedTest1(t *testing.T) {
 			}
 		}
 	default:
-		if out, err := shell(true, "go", "get", defaultLibc+"@master"); err != nil { //TODO- &master
+		if out, err := shell(true, "go", "get", *oLibc+"@latest"); err != nil {
 			t.Fatalf("%s\vFAIL: %v", out, err)
 		}
 	}
@@ -1410,7 +1390,7 @@ func testSQLiteSpeedTest1(t *testing.T) {
 
 	executable := "./speedtest1"
 	if runtime.GOOS == "windows" {
-		executable = "speedtest1.exe"
+		executable = "./speedtest1.exe"
 	}
 	args := []string{"build"}
 	if s := *oXTags; s != "" {
