@@ -31,6 +31,10 @@ const (
 	objectPkg
 )
 
+var (
+	builtinsObject = &object{}
+)
+
 type object struct {
 	defs           map[string]gc.Node // extern: node
 	externs        nameSet            // CAPI
@@ -43,7 +47,9 @@ type object struct {
 
 	kind int // {objectFile, objectPkg}
 
-	imported bool
+	imported    bool
+	isExtracted bool // from an .a file
+	isRequired  bool // from an .a file and needed
 }
 
 func newObject(kind int, id string) *object {
@@ -54,6 +60,15 @@ func newObject(kind int, id string) *object {
 	}
 }
 
+func (o *object) requiredFor(nm string) {
+	o.isRequired = true
+	// if dmesgs {
+	// 	if o.isExtracted {
+	// 		dmesg("object %q.isRequired for %q", o.id, nm)
+	// 	}
+	// }
+}
+
 func (o *object) load() (file *gc.SourceFile, err error) {
 	if o.kind == objectPkg {
 		return nil, errorf("object.load: internal error: wrong kind")
@@ -99,7 +114,7 @@ func (o *object) collectExternVars(file *gc.SourceFile) (seps map[*gc.VarSpec]st
 
 // link name -> type ID
 func (o *object) collectTypes(file *gc.SourceFile) (types map[string]string, err error) {
-	// trc("==== %s (%v:)", o.id, origin(0))
+	// tq	rc("==== %s (%v:)", o.id, origin(0))
 	var a []string
 	in := map[string]gc.Node{}
 	for _, decl := range file.TopLevelDecls {
@@ -149,7 +164,7 @@ func (o *object) collectConsts(file *gc.SourceFile) (consts map[string]string, e
 
 					var b strings.Builder
 					if assert && len(spec.ExprList) == 0 {
-						panic(todo("%q", x.Source(false)))
+						panic(todo("%v: %q", file.EOF.Position().Filename, x.Source(false)))
 					}
 
 					b.Write(spec.ExprList[i].Expr.Source(true))
@@ -169,6 +184,15 @@ func (o *object) collectConsts(file *gc.SourceFile) (consts map[string]string, e
 }
 
 func (t *Task) link() (err error) {
+	// 	if dmesgs {
+	// 		dmesg("%v: t.linkFiles %v", origin(1), t.linkFiles)
+	// 		defer func() {
+	// 			if err != nil {
+	// 				dmesg("", errorf("", err))
+	// 			}
+	//		}()
+	//	}
+
 	if len(t.inputFiles)+len(t.linkFiles) == 0 {
 		return errorf("no input files")
 	}
@@ -195,27 +219,63 @@ func (t *Task) link() (err error) {
 
 	fset := token.NewFileSet()
 	objects := map[string]*object{}
-	mode := os.Getenv("GO111MODULE")
 	var libc *object
+	var linkFiles []string
 	for _, v := range t.linkFiles {
+		// if dmesgs {
+		// 	dmesg("%v: link file %s", origin(1), v)
+		// }
 		var object *object
 		switch {
 		case strings.HasPrefix(v, "-l="):
-			object, err = t.getPkgSymbols(v[len("-l="):], mode)
-			if err != nil {
-				break
-			}
+			for _, prefix := range t.L {
+				switch {
+				case strings.HasPrefix(prefix, "/"): // -L/foo/bar
+					continue
+				case strings.HasPrefix(prefix, "."): // -L.
+					continue
+				case strings.Contains(prefix, "/"):
+					a := strings.Split(prefix, "/")
+					if !strings.Contains(a[0], ".") { // -Lfoo/bar
+						continue
+					}
 
-			if object.pkgName == "libc" && libc == nil {
-				libc = object
+					// -Lexample.com/foo
+				}
+
+				lib := "lib" + v[len("-l="):]
+				ip := prefix + "/" + lib
+				if prefix == defaultLibs && lib == "libc" {
+					ip = t.libc
+				}
+				object, err = t.getPkgSymbols(ip)
+				if err == nil {
+					if object.pkgName == "libc" && libc == nil {
+						libc = object
+					}
+					break
+				}
 			}
 		default:
 			object, err = t.getFileSymbols(fset, v)
 		}
 		if err != nil {
+			if t.isExeced {
+				// if dmesgs {
+				// 	dmesg("%q: ignoring %v", v, err)
+				// }
+				continue
+			}
+
 			return err
 		}
 
+		if object == nil {
+			continue
+		}
+
+		_, object.isExtracted = t.archiveLinkFiles[v]
+		linkFiles = append(linkFiles, v)
 		if _, ok := objects[v]; !ok {
 			objects[v] = object
 		}
@@ -239,22 +299,26 @@ func (t *Task) link() (err error) {
 			return err
 		}
 
-		return l.link(t.o, t.linkFiles, objects)
+		return l.link(t.o, linkFiles, objects)
 	default:
-		return errorf("TODO %v %v %v %v", t.args, t.inputFiles, t.compiledfFiles, t.linkFiles)
+		return errorf("TODO t.o %q, t.args %q, t.inputFiles %q,  t.compiledfFiles %q, t.linkFiles %q", t.o, t.args, t.inputFiles, t.compiledfFiles, t.linkFiles)
 	}
 }
 
-func (t *Task) getPkgSymbols(importPath, mode string) (r *object, err error) {
-	switch mode {
-	case "", "on":
-		// ok
-	default:
-		if !isTesting {
-			return nil, errorf("GO111MODULE=%s not supported", mode)
-		}
+func (t *Task) getPkgSymbols(importPath string) (r *object, err error) {
+	if dmesgs {
+		dmesg("==== import %q t.goos=%v t.goarch=%v", importPath, t.goos, t.goarch)
 	}
-
+	// if dmesgs {
+	// 	defer func() {
+	// 		switch {
+	// 		case r != nil:
+	// 			dmesg("lib importPath %q: (%q, %v)", importPath, r.id, err)
+	// 		default:
+	// 			dmesg("lib importPath %q: (%p, %v)", importPath, r, err)
+	// 		}
+	// 	}()
+	// }
 	pkgs, err := packages.Load(
 		&packages.Config{
 			Mode: packages.NeedFiles,
@@ -271,7 +335,7 @@ func (t *Task) getPkgSymbols(importPath, mode string) (r *object, err error) {
 	}
 
 	pkg := pkgs[0]
-	if len(pkg.Errors) != 0 {
+	if len(pkg.Errors) != 0 && !t.ignoreLinkErrors {
 		var a []string
 		for _, v := range pkg.Errors {
 			a = append(a, v.Error())
@@ -281,6 +345,9 @@ func (t *Task) getPkgSymbols(importPath, mode string) (r *object, err error) {
 
 	r = newObject(objectPkg, importPath)
 	for _, fn := range pkg.GoFiles {
+		// if dmesgs {
+		// 	dmesg("importing file %q", fn)
+		// }
 		b, err := os.ReadFile(fn)
 		if err != nil {
 			return nil, errorf("%s: %v", importPath, err)
@@ -301,6 +368,9 @@ func (t *Task) getPkgSymbols(importPath, mode string) (r *object, err error) {
 					break
 				}
 
+				// if dmesgs {
+				// 	dmesg("imported func %q", x.FunctionName.Src())
+				// }
 				r.externs.add(x.FunctionName.Src())
 			case *gc.VarDecl:
 				for _, v := range x.VarSpecs {
@@ -310,6 +380,9 @@ func (t *Task) getPkgSymbols(importPath, mode string) (r *object, err error) {
 							continue
 						}
 
+						// 						if dmesgs {
+						// 							dmesg("imported var %q", nm)
+						// 						}
 						r.externs.add(nm)
 					}
 				}
@@ -478,10 +551,10 @@ type externVar struct {
 }
 
 type linker struct {
+	aliases               map[string]string // alias: canonical
 	errors                errors
 	externVars            map[string]*externVar // key: linkname
 	externs               map[string]*object
-	weakAliases           map[string]string // redirect what link name: redirect to link name
 	fileLinkNames2GoNames dict
 	fileLinkNames2IDs     dict
 	forceExternalPrefix   nameSet
@@ -490,6 +563,7 @@ type linker struct {
 	goTags                []string
 	imports               []*object
 	importsByPath         map[string]*object
+	libAliases            map[string]string // linkname: aka linkname, generate var aka = name for aliased functions
 	libc                  *object
 	maxUintptr            uint64
 	out                   io.Writer
@@ -507,7 +581,9 @@ type linker struct {
 	undefsReported        nameSet
 	unsafeName            string
 
-	closed bool
+	closed      bool
+	keepStrings bool
+	needFpFunc  bool
 }
 
 func newLinker(task *Task, libc *object) (*linker, error) {
@@ -560,18 +636,20 @@ func newLinker(task *Task, libc *object) (*linker, error) {
 		maxUintptr = math.MaxUint32
 	}
 	return &linker{
+		aliases:        map[string]string{},
 		externVars:     map[string]*externVar{},
 		externs:        map[string]*object{},
 		fset:           token.NewFileSet(),
 		goTags:         goTags[:],
 		importsByPath:  map[string]*object{},
+		keepStrings:    task.keepStrings,
+		libAliases:     map[string]string{},
 		libc:           libc,
 		maxUintptr:     maxUintptr,
 		stringLiterals: map[string]int64{},
 		synthDecls:     map[string][]byte{},
 		task:           task,
 		tldTypes:       map[string]struct{ linkName, goName string }{},
-		weakAliases:    map[string]string{},
 	}, nil
 }
 
@@ -593,42 +671,109 @@ func (l *linker) w(s string, args ...interface{}) {
 	}
 }
 
-func (l *linker) link(ofn string, linkFiles []string, objects map[string]*object) (err error) {
-	if dmesgs {
-		dmesg("link(%q, %q)", ofn, linkFiles)
+func (l *linker) registerLibAliases(obj *object) error {
+	for aka, name := range obj.meta.Aliases {
+		// eg. "XFcPatternDestroy": "XIA__FcPatternDestroy"
+		visibility := obj.meta.Visibility[aka]
+		// eg. "XFcPatternDestroy": "default"
+		if visibility != "default" {
+			continue
+		}
+
+		ex := l.libAliases[name]
+		if ex != "" && ex != aka {
+			return errorf("conflicting function alias %q: %q and %q", aka, ex, name)
+		}
+
+		l.libAliases[name] = aka
+		// if dmesgs && {
+		// 	dmesg("ALIASED[%q]=%q", aka, name)
+		// }
 	}
-	// Force a link error for things not really supported or that only panic at runtime.
+	return nil
+}
+
+func (l *linker) link(ofn string, linkFiles []string, objects map[string]*object) (err error) {
+	//	if dmesgs {
+	//		dmesg("packageName=%v inputFiles=%v inputArchives=%v linkFiles=%v", l.task.packageName, l.task.inputFiles, l.task.inputArchives, linkFiles)
+	// 		dmesg("link(%q, %q)", ofn, linkFiles)
+	// 		defer func() {
+	// 			if err != nil {
+	// 				dmesg("", errorf("", err))
+	// 			}
+	//		}()
+	//	}
+
+	// ccgo -o libfoo.go libfoo.a
+	handleLibAliases := (l.task.packageName != "" && l.task.packageName != "main") && len(l.task.inputFiles) == 0 && len(l.task.inputArchives) == 1
+
+	//TODO Force a link error for things not really supported or that only panic at runtime.
 	var tld nameSet
 	// Build the symbol table. First try normal definitions.
 	for _, linkFile := range linkFiles {
 		object := objects[linkFile]
+		if handleLibAliases {
+			if err := l.registerLibAliases(object); err != nil {
+				return err
+			}
+		}
+
+		// 		if dmesgs {
+		// 			dmesg("checking object.id=%s", object.id)
+		// 		}
 		for nm := range object.externs { // object defines nm
+			// 			if dmesgs {
+			// 				dmesg("extern %s declared in %s", nm, object.id)
+			// 			}
 			if _, ok := l.externs[nm]; !ok { // extern is unresolved
 				l.externs[nm] = object
-				if dmesgs {
-					dmesg("extern %s resolved in %s", nm, object.id)
-				}
+				object.requiredFor(nm)
+				// 				if dmesgs {
+				// 					dmesg("extern %s resolved in %s", nm, object.id)
+				// 				}
 			}
 			tld.add(nm)
 		}
 	}
+	// Then try aliases
+	for _, linkFile := range linkFiles {
+		object := objects[linkFile]
+		for alias, canonical := range object.meta.Aliases { // object defines an alias for canonical
+			// 			if dmesgs {
+			// 				dmesg("extern %s alias in %s", nm, object.id)
+			// 			}
+			if _, ok := l.externs[alias]; !ok { // extern is still unresolved
+				l.externs[alias] = object
+				object.requiredFor(alias)
+				l.aliases[alias] = canonical
+				// 				if dmesgs {
+				// 					dmesg("extern %s alias resolved in %s", nm, object.id)
+				// 				}
+			}
+			tld.add(alias)
+		}
+	}
 	// Then try weak aliases
 	for _, linkFile := range linkFiles {
 		object := objects[linkFile]
-		for nm, to := range object.meta.WeakAliases { // object defines a weak alias for nm
-			if _, ok := l.externs[nm]; !ok { // extern is still unresolved
-				l.externs[nm] = object
-				l.weakAliases[nm] = to
-				if dmesgs {
-					dmesg("extern %s weak resolved in %s", nm, object.id)
-				}
+		for alias, canonical := range object.meta.WeakAliases { // object defines a weak alias for canonical
+			// 			if dmesgs {
+			// 				dmesg("extern %s weak alias in %s", nm, object.id)
+			// 			}
+			if _, ok := l.externs[alias]; !ok { // extern is still unresolved
+				l.externs[alias] = object
+				object.requiredFor(alias)
+				l.aliases[alias] = canonical
+				// 				if dmesgs {
+				// 					dmesg("extern %s weak alias resolved in %s", nm, object.id)
+				// 				}
 			}
-			tld.add(nm)
+			tld.add(alias)
 		}
 	}
 	l.tld.registerNameSet(l, tld, true)
-	l.textSegmentNameP = l.tld.reg.put("ts")
-	l.textSegmentName = l.tld.reg.put("ts")
+	l.textSegmentNameP = l.tld.reg.put(ccgoTS)
+	l.textSegmentName = l.tld.reg.put(ccgoTS)
 	l.reflectName = l.tld.reg.put("reflect")
 	l.runtimeName = l.tld.reg.put("runtime")
 	l.unsafeName = l.tld.reg.put("unsafe")
@@ -640,7 +785,12 @@ func (l *linker) link(ofn string, linkFiles []string, objects map[string]*object
 	}
 	var undefs []undef
 	for _, linkFile := range linkFiles {
-		switch object := objects[linkFile]; {
+		object := objects[linkFile]
+		if object.isExtracted && !object.isRequired {
+			continue
+		}
+
+		switch {
 		case object.kind == objectFile:
 			file, err := object.load()
 			if err != nil {
@@ -655,7 +805,15 @@ func (l *linker) link(ofn string, linkFiles []string, objects map[string]*object
 				lib, ok := l.externs[nm]
 				if !ok {
 					// trc("%q %v: %q", object.id, pos, nm)
-					undefs = append(undefs, undef{pos, nm})
+					switch r := l.rawName(nm); {
+					case strings.HasPrefix(r, "__builtin_"):
+						l.externs[nm] = builtinsObject
+					default:
+						// 						if dmesgs {
+						// 							dmesg("extern %s NOT resolved", nm)
+						// 						}
+						undefs = append(undefs, undef{pos, nm})
+					}
 					continue
 				}
 
@@ -676,7 +834,7 @@ func (l *linker) link(ofn string, linkFiles []string, objects map[string]*object
 			}
 		}
 	}
-	if len(undefs) != 0 && !l.task.header {
+	if len(undefs) != 0 && !l.task.header && !l.task.ignoreLinkErrors {
 		sort.Slice(undefs, func(i, j int) bool {
 			a, b := undefs[i].pos, undefs[j].pos
 			if a.Filename < b.Filename {
@@ -736,16 +894,6 @@ func (l *linker) link(ofn string, linkFiles []string, objects map[string]*object
 			l.w("\n\t%s \"reflect\"", nm)
 		}
 		rtDummy := ""
-		if l.task.experimentPin != 0 {
-			switch nm := l.runtimeName; nm {
-			case "runtime":
-				l.w("\n\t\"runtime\"")
-				rtDummy = "\t_ runtime.Pinner\n"
-			default:
-				l.w("\n\t%s \"runtime\"", nm)
-				rtDummy = fmt.Sprintf("\t_ %s.Pinner\n", nm)
-			}
-		}
 		switch nm := l.unsafeName; nm {
 		case "unsafe":
 			l.w("\n\t\"unsafe\"")
@@ -786,7 +934,11 @@ var (
 
 	for _, linkFile := range linkFiles {
 		object := objects[linkFile]
-		if object.kind != objectFile {
+		if object == builtinsObject || object.kind != objectFile {
+			continue
+		}
+
+		if object.isExtracted && !object.isRequired {
 			continue
 		}
 
@@ -883,7 +1035,9 @@ var (
 						l.print0(&b, fi, spec)
 						l.externVars[linkName] = &externVar{rendered: b.bytes(), spec: spec}
 					default:
-						return errorf("loading %s: multiple definitions of %s", object.id, l.rawName(linkName))
+						if !l.task.isExeced {
+							return errorf("loading %s: multiple definitions of %s", object.id, l.rawName(linkName))
+						}
 					}
 				default:
 					fi := l.newFnInfo(spec)
@@ -948,11 +1102,27 @@ var (
 				l.print0(&b, fi, n)
 				l.synthDecls[nm2] = b.bytes()
 			case *gc.FunctionDecl:
-				if ln := x.FunctionName.Src(); l.isMeta(x, ln) || l.task.header {
+				ln := x.FunctionName.Src()
+				if l.isMeta(x, ln) || l.task.header {
 					break
 				}
 
 				l.funcDecl(x)
+				if len(l.libAliases) == 0 {
+					break
+				}
+
+				if !strings.HasPrefix(ln, tag(external)) {
+					break
+				}
+
+				aliasNm := l.libAliases[ln]
+				if aliasNm == "" || !strings.HasPrefix(aliasNm, tag(external)) {
+					break
+				}
+
+				// trc("ALIAS %q -> %q", aliasNm, ln)
+				l.w("\n\nvar %s = %s\n\n", aliasNm, ln)
 			default:
 				l.err(errorf("TODO %T", x))
 			}
@@ -1001,7 +1171,7 @@ func (l *linker) postProcess(fn string, b []byte) (r []byte) {
 			inFunc = true
 		case strings.HasPrefix(line, "}"):
 			inFunc = false
-		case s == "", s == ";":
+		case s == "":
 			if inFunc {
 				continue
 			}
@@ -1040,7 +1210,6 @@ func (l *linker) isMeta(n gc.Node, linkName string) bool {
 }
 
 func (l *linker) funcDecl(n *gc.FunctionDecl) {
-	l.w("\n\n")
 	info := l.newFnInfo(n)
 	var static []gc.Node
 	w := 0
@@ -1058,6 +1227,7 @@ func (l *linker) funcDecl(n *gc.FunctionDecl) {
 		case *gc.FunctionLit:
 			l.w("func init() ")
 			l.print(info, x.FunctionBody)
+			l.w("\n\n")
 		default:
 			l.print(info, v)
 		}
@@ -1155,13 +1325,15 @@ func (l *linker) epilogue() {
 		return
 	}
 
-	l.w(`
+	if l.needFpFunc {
+		l.w(`
 
 func %s(f interface{}) uintptr {
 	type iface [2]uintptr
 	return (*iface)(unsafe.Pointer(&f))[1]
 }
 `, ccgoFP)
+	}
 	var a []string
 	for k := range l.externVars {
 		a = append(a, k)
@@ -1180,19 +1352,22 @@ func %s(f interface{}) uintptr {
 }
 
 func (l *linker) prologue(nm string) {
-	l.w(`// %s%s/%s by '%s %s'%s.
+	// 	if dmesgs {
+	// 		dmesg("DBG t@%p.buildLines = %q", l.task, l.task.buildLines)
+	// 	}
+	l.w(`// %s%s/%s by '%s %s'%s
 
-//go:build %[2]s && %[3]s
-// +build %[2]s,%[3]s
+%s
 
-package %[7]s
+package %s
 
 `,
 		generatedFilePrefix,
 		l.task.goos, l.task.goarch,
-		filepath.Base(l.task.args[0]),
-		strings.Join(l.task.args[1:], " "),
+		safeArg(filepath.Base(l.task.args[0])),
+		strings.Join(safeArgs(l.task.args[1:]), " "),
 		generatedFileSuffix,
+		l.task.buildLines,
 		nm,
 	)
 }
@@ -1228,7 +1403,9 @@ func (l *linker) newFnInfo(n gc.Node) (r *fnInfo) {
 					r.linkNames.add(nm)
 				}
 			case gc.STRING_LIT:
-				r.linker.stringLit(tok.Src(), true)
+				if !l.keepStrings {
+					r.linker.stringLit(tok.Src(), true)
+				}
 			}
 		}, nil)
 	}
@@ -1251,7 +1428,11 @@ func (fi *fnInfo) name(linkName string) string {
 		}
 
 		r := fi.linker.rawName(linkName)
-		if fi.linker.undefsReported.add(r) {
+		if strings.HasPrefix(r, "__builtin_") {
+			return fmt.Sprintf("%s%s%s", fi.linker.task.tlsQualifier, fi.linker.task.prefixExternal, r)
+		}
+
+		if !fi.linker.task.ignoreLinkErrors && fi.linker.undefsReported.add(r) {
 			fi.linker.err(errorf("undefined: %q %v", r, symKind(linkName)))
 		}
 		return fi.linker.task.prefixUndefined + r
@@ -1268,6 +1449,11 @@ func (fi *fnInfo) name(linkName string) string {
 	case importQualifier:
 		switch nm := linkName[len(tag(importQualifier)):]; nm {
 		case "libc":
+			if fi.linker.libc == nil {
+				fi.linker.err(errorf("TODO %q %v - no libc object", linkName, symKind(linkName)))
+				return linkName
+			}
+
 			return fi.linker.libc.qualifier
 		case "runtime":
 			return fi.linker.runtimeName
@@ -1325,6 +1511,9 @@ func (l *linker) print0(w writer, fi *fnInfo, n interface{}) {
 		switch x.Ch {
 		case gc.IDENTIFIER:
 			id := x.Src()
+			if id == tag(preserve)+ccgoFP {
+				l.needFpFunc = true
+			}
 			nm := fi.name(id)
 			if nm == "" {
 				w.w("%s", id)
@@ -1340,20 +1529,14 @@ func (l *linker) print0(w writer, fi *fnInfo, n interface{}) {
 			if obj == nil {
 				r := l.rawName(nm)
 				w.w("%s%s", l.task.prefixUndefined, r)
-				if l.undefsReported.add(r) {
+				if !l.task.ignoreLinkErrors && l.undefsReported.add(r) {
 					l.err(errorf("%v: undefined: %q %v", x.Position(), r, symKind(nm)))
 				}
 				return
 			}
 
-			if to := l.weakAliases[id]; to != "" {
-				if dmesgs {
-					dmesg("redirect id %q nm %q to %q", id, nm, to)
-				}
+			if to := l.aliases[id]; to != "" {
 				nm = fi.name(to)
-				if dmesgs {
-					dmesg("new nm %q", nm)
-				}
 			}
 			if obj.kind == objectPkg {
 				w.w("%s.%s", obj.qualifier, nm)
@@ -1362,6 +1545,11 @@ func (l *linker) print0(w writer, fi *fnInfo, n interface{}) {
 
 			w.w("%s", nm)
 		case gc.STRING_LIT:
+			if l.keepStrings {
+				w.w("%s", x.Src())
+				break
+			}
+
 			w.w("%s", l.stringLit(x.Src(), false))
 		default:
 			w.w("%s", x.Src())
@@ -1369,6 +1557,15 @@ func (l *linker) print0(w writer, fi *fnInfo, n interface{}) {
 		return
 	}
 
+	if x, ok := n.(*gc.BinaryExpr); ok {
+		switch x.Op.Ch {
+		case gc.LOR:
+			if bytes.Equal(x.A.Source(false), x.B.Source(false)) {
+				n = x.A
+			}
+		}
+	}
+
 	t := reflect.TypeOf(n)
 	v := reflect.ValueOf(n)
 	var zero reflect.Value
